;-------------------------------------------------------------------------------
;  Copyright (c) 2000 by Trenz Electronic.
;  Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
;      
;  This program is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation; either version 2 of the License, or
;  (at your option) any later version.
;      
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;      
;  You should have received a copy of the GNU General Public License
;  along with this program; if not, write to the Free Software
;  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;-------------------------------------------------------------------------------
;  Project:      Full-Speed USB 1.1 Function Controller
;  File:         firmware.a51
;  Description:  usbSIE reference application, firmware.
;  Version:      FB, 2000jun21
;-------------------------------------------------------------------------------

	$MOD51

STACKTOP EQU 70H                                ; start of stack (grows up)

ledSL       EQU 0F000h                          ; left  7-Segment display
ledSR       EQU 0F001h                          ; right 7-Segment display
ledB        EQU 0F002h                          ; bargraph display

ledREG0     EQU 00h
ledREG1     EQU 01h
ledREG2     EQU 02h
ledREG3     EQU 03h
ledREG4     EQU 04h
ledREG5     EQU 05h
ledREG6     EQU 06h
ledREG7     EQU 07h
ledREG8     EQU 08h
ledREG9     EQU 09h
ledREGa     EQU 0Ah
ledREGb     EQU 0Bh
ledREGc     EQU 0Ch
ledREGd     EQU 0Dh
ledREGe     EQU 0Eh
ledREGf     EQU 0Fh

;ledREG0     EQU 77h                             ; "0"
;ledREG1     EQU 12h                             ; "1"
;ledREG2     EQU 5Eh                             ; "2"

usbFIFO     EQU 0E000h                          ; usb fifos
usbFIFOlen  EQU 8                               ; usb fifo depth

usbCTRL       EQU 0D000h                        ; usb endpoint #0 control
usbCTRLread   EQU 80h                           ; control read sequence
usbCTRLwrite  EQU 40h                           ; control write sequence
usbCTRLstatus EQU 20h                           ; status acknowledge
usbCTRLstall  EQU 10h                           ; STALL endpoint

usbSTAT       EQU 0D000h                        ; usb endpoint #0 status
usbSTATsetup  EQU 80h                           ; setup stage
usbSTATdata   EQU 40h                           ; data stage
usbSTATstatus EQU 20h                           ; status stage
usbSTATall    EQU 0E0h                          ; any status bit

usbDADX     EQU 0C000h                          ; usb device address

bmRequestType EQU usbFIFO
bRequest      EQU usbFIFO + 1
wValueLo      EQU usbFIFO + 2
wValueHi      EQU usbFIFO + 3
wIndexLo      EQU usbFIFO + 4
wIndexHi      EQU usbFIFO + 5
wLengthLo     EQU usbFIFO + 6
wLengthHi     EQU usbFIFO + 7

;-------------------------------------------------------------------------------
	DSEG	AT 20H

FLAGS:         DS  1                            ; This register is bit-addressable
  Configured   EQU FLAGS.0	                    ; Is this device configured
  STALL		   EQU FLAGS.1	                    ; Need to STALL endpoint 0
  SendData	   EQU FLAGS.2                      ; Need to send data to PC Host
  IsDescriptor EQU FLAGS.3                      ; Enable a shortcut reply

Temp:
Temp1:         DS 1                             ; A temporary working register
Temp2:         DS 1                             ; A temporary working register
Temp3:         DS 1                             ; A temporary working register
Temp4:         DS 1                             ; A temporary working register
Temp5:         DS 1                             ; A temporary working register
Temp6:         DS 1                             ; A temporary working register
Temp7:         DS 1                             ; A temporary working register

ConfigVal:     DS 1                             ; configuration value
EpHalt:        DS 1                             ; endpoint HALT feature

	DSEG
	ORG 30H
;CNTR:          DS 1                             ; counter for wait routine

;-------------------------------------------------------------------------------
	CSEG
	ORG 0000H                                   ; program starts at 0 after reset

start:
	MOV  SP, #STACKTOP                          ; initialize stack pointer ...

	;-- set device address to zero
	MOV  A, #00h
	MOV  DPTR, #usbDADX
	MOVX @DPTR, A

waitSETUP:
	;-- display "0"
	;MOV  A, #ledREG0
	;MOV  DPTR, #ledREG
	;MOVX @DPTR, A

	;-- wait for SETUP transaction
	MOV  DPTR, #usbSTAT
w1:	MOVX A, @DPTR
	ANL  A, #usbSTATall
	CJNE A, #usbSTATsetup, w1

	;-- display "1"
	;MOV  A, #ledREG1
	;MOV  DPTR, #ledREG
	;MOVX @DPTR, A

	CALL Dispatch

	JMP waitSETUP

;-------------------------------------------------------------------------------
;* * *   d i s p a t c h   c h a p t e r   9   r e q u e s t s   * * * * * * * *
;-------------------------------------------------------------------------------
Dispatch:
	;-------------------------------------------
	; Reference: USB Serial Bus Specification 1.1
	;            Chapter 9.3 USB Device Requests
	;   bmRequestType
	;     (7) Data transfer direction
	;        0= Host-to-device
	;        1= Device-to-host
	;     (6-5) Type
	;        00= Standard
	;        01= Class
	;        10= Vendor
	;        11= Reserved
	;     (4-0) Recipient
	;        00000= Device
	;        00001= Interface
	;        00010= Endpoint
	;        00011= Other
	;        All other codes are reserved
	;-------------------------------------------
	; Reference: USB Design by Example, 
	;            John Hyde, Wiley Computer Publishing,
	;            ISBN 0-471-37048-7
	;            Chapter 6 Buttons & Lights, Example 1-Step 3: 
	;                       Implement Microcontroller Code
	;
	;   with bmRequest(5) select
	;   cc= "11"                      when '1',
	;       bmRequestType(1 downto 0) when others;
	;   rr= bRequest(3 downto 0);
	;   code= cc & rr;
	;
	;   Note: cc  recipient
	;         00= device
	;         01= interface
	;         10= endpoint
	;         11= class
	;- - - - - - - - - - - - - - - - - - - - - -
	; rr device            interface         endpoint        class
	; 0  devGET_STATUS     ifGET_STATUS      epGET_STATUS    ---
	; 1  devCLEAR_FEATURE  ifCLEAR_FEATURE   epCLEAR_FEATURE xxxGET_REPORT
	; 2  ---               ---               ---             xxxGET_IDLE
	; 3  devSET_FEATURE    ifSET_FEATURE     epSET_FEATURE   xxxGET_PROTOCOL
	; 4  ---               ---               ---             ---
	; 5  SET_ADDRESS       ---               ---             ---
	; 6  GET_DESCRIPTOR    xxxGET_DESCRIPTOR ---             ---
	; 7  SET_DESCRIPTOR    xxxSET_DESCRIPTOR ---             ---
	; 8  GET_CONFIGURATION ---               ---             ---
	; 9  SET_CONFIGURATION ---               ---             xxxSET_REPORT
	; a  ---               GET_INTERFACE     ---             xxxSET_IDLE
	; b  ---               SET_INTERFACE     ---             xxxSET_PROTOCOL
	; c  ---               ---               epSYNC_FRAME
	;-------------------------------------------

	MOV  DPTR, #bmRequestType
	MOVX A, @DPTR

	MOV  C, ACC.7		                        ; Data transfer direction
 	MOV  SendData, C

	ANL	 A, #01011100b
	JNZ  BadRequest                             ; reserved code

	MOVX A, @DPTR
	MOV  C, ACC.0
	ANL  C, ACC.1
	JC   BadRequest                             ; recipient= other

	JNB	 ACC.5, n5                              ; bmRequestType(5)
	MOV  A, #00000011b                          ; cc= "11"
n5:	ANL  A, #00000011b                          ; cc= bmRequestType(1 downto 0)

	SWAP A
	MOV  Temp, A                                ; Save HI nibble of rr

	INC  DPTR
	MOVX A, @DPTR                               ; bRequest

	ANL	A, #00001111b                           ; bRequest(3 downto 0)
 	ORL	A, Temp                                 ; this is rr!

	MOV  DPTR, #CommandTable
	CALL BumpDPTR		                        ; point to entry

	MOVX A, @DPTR                               ; get the offset
	MOV  DPTR, #CommandTable
	CALL BumpDPTR                               ; get routine address

	PUSH DPL                                    ; create return address on stack
	PUSH DPH                                    ; Note: JMP @A+DPTR not used since A, DPTR needed
	RET                                         ; go to service routine

BadRequest:
	JMP  invalid

BumpDPTR:                                       ; Returns (DPTR + ACC)
	ADD  A, DPL
	MOV  DPL, A
	JNC  nc
	INC  DPH                                    ; Need 16 bit arithmetic here
nc:	RET

; Since the table only contains byte offsets, it is important that all these routines are
; within one page (100H) of CommandTable
CommandTable:
	;-- device commands
	DB devGET_STATUS     - CommandTable         ; 0
	DB devCLEAR_FEATURE  - CommandTable         ; 1
	DB invalid           - CommandTable         ; 2
	DB devSET_FEATURE    - CommandTable         ; 3
	DB invalid           - CommandTable         ; 4
	DB SET_ADDRESS       - CommandTable         ; 5
	DB GET_DESCRIPTOR    - CommandTable         ; 6
	DB SET_DESCRIPTOR    - CommandTable         ; 7
	DB GET_CONFIGURATION - CommandTable         ; 8
	DB SET_CONFIGURATION - CommandTable         ; 9
	DB invalid           - CommandTable         ; 10
	DB invalid           - CommandTable         ; 11
	DB invalid           - CommandTable         ; 12
	DB invalid           - CommandTable         ; 13
	DB invalid           - CommandTable         ; 14
	DB invalid           - CommandTable         ; 15

	;-- interface commands
	DB ifGET_STATUS      - CommandTable         ;
	DB ifCLEAR_FEATURE   - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB ifSET_FEATURE     - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB xxxGET_DESCRIPTOR - CommandTable         ;
	DB xxxSET_DESCRIPTOR - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB GET_INTERFACE     - CommandTable         ;
	DB SET_INTERFACE     - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;

	;-- endpoint commands
	DB epGET_STATUS      - CommandTable         ;
	DB epCLEAR_FEATURE   - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB epSET_FEATURE     - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB epSYNCH_FRAME     - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;

	;-- class requests
	DB invalid           - CommandTable         ;
	DB xxxGET_REPORT     - CommandTable         ;
	DB xxxGET_IDLE       - CommandTable         ;
	DB xxxGET_PROTOCOL   - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB xxxSET_REPORT     - CommandTable         ;
	DB xxxSET_IDLE       - CommandTable         ;
	DB xxxSET_PROTOCOL   - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;
	DB invalid           - CommandTable         ;

;-------------------------------------------------------------------------------
;* * *   s t a n d a r d   d e v i c e   r e q u e s t s   * * * * * * * * * * *
;-------------------------------------------------------------------------------

;*** 9.4.1 Clear Feature *******************************************************
epCLEAR_FEATURE:
	MOV  DPTR, #wValueLo
	MOVX A, @DPTR                               ; endpoint

	;-- todo: store HALT info for every endpoint
	MOV  EpHalt, #0
	
	JMP  NoDataControl

;*** 9.4.2 Get Configuration ***************************************************
GET_CONFIGURATION:
	;-- put config val into fifo
	MOV  DPTR, #usbFIFO
	MOV  A, ConfigVal
	MOVX @DPTR, A

	MOV  A, #1
	JMP  ControlRead

;*** 9.4.3 Get Descriptor ******************************************************
GET_DESCRIPTOR:

	MOV  DPTR, #wValueHI
	MOVX A, @DPTR

	DEC  A
	JZ   devGET_DESCRIPTOR                      ; wValueLO= 1

	DEC  A
	JZ   cfgGET_DESCRIPTOR                      ; wValueLO= 2

	DEC  A
	JZ   strGET_DESCRIPTOR                      ; wValueLO= 3

	JMP  invalid

;*** 9.4.5 Get Status **********************************************************
devGET_STATUS:
	MOV  A, #1                                  ; self powered
	JMP  doGET_STATUS

ifGET_STATUS:
	MOV  A, #0                                  ; reserved (zero)
	JMP  doGET_STATUS

epGET_STATUS:
	MOV  A, EpHalt                              ; endpoint HALT
	JMP doGET_STATUS

;*** 9.4.6 Set Address *********************************************************
SET_ADDRESS:
	MOV  DPTR, #wValueLo
	MOVX A, @DPTR

	;-- set device address (deferred until next SETUP transaction)
	MOV  DPTR, #usbDADX
	MOVX @DPTR, A

	JMP  NoDataControl

;*** 9.4.7 Set Configuration ***************************************************
SET_CONFIGURATION:
	;-- save config value
	MOV  DPTR, #wValueLo
	MOVX A, @DPTR
	MOV  ConfigVal, A

	JMP  NoDataControl

;*** 9.4.9 Set Feature *********************************************************
epSET_FEATURE:
	MOV  DPTR, #wValueLo
	MOVX A, @DPTR                               ; endpoint

	;-- todo: store HALT info for every endpoint
	MOV  EpHalt, #1
	
	JMP  NoDataControl

;-------------------------------------------------------------------------------
;* * *   H I D   s p e c i f i c   r e q u e s t s   * * * * * * * * * * * * * *
;-------------------------------------------------------------------------------

;*** HID 7.1.1 Get Descriptor **************************************************
xxxGET_DESCRIPTOR:
	MOV  DPTR, #wValueHi
	MOVX A, @DPTR

	CLR  C
	SUBB A, #21h
	JZ   hidGET_DESCRIPTOR

	DEC  A
	JZ   rptGET_DESCRIPTOR

	JMP  invalid

;*** HID 7.2.1 Get Report ******************************************************
xxxGET_REPORT:
	JMP  doGET_REPORT
	;JMP  ControlRead

;*** HID 7.2.2 Set Report ******************************************************
xxxSET_REPORT:

	;-- control-write sequence
sr1:
	MOV  DPTR, #usbCTRL
	MOV  A, #usbCTRLwrite
	MOVX @DPTR, A

	;-- wait for next transaction
	MOV  DPTR, #usbSTAT
sr2:	
	MOVX A, @DPTR
	ANL  A, #usbSTATall
	JZ   sr2

	;-- discard data stage
	CJNE A, #usbSTATstatus, sr1

	JMP  doSET_REPORT
	;JMP  AcknowledgeStatus

;-------------------------------------------------------------------------------
;* * *   i n v a l i d   r e q u e s t s   * * * * * * * * * * * * * * * * * * *
;-------------------------------------------------------------------------------
SET_DESCRIPTOR:                                 ; descriptors are static
SET_INTERFACE:                                  ; only one Interface
GET_INTERFACE:                                  ; no alternate setting
devSET_FEATURE:                                 ; no features to be set or cleared
ifSET_FEATURE:                                  ; no features to be set or cleared
devCLEAR_FEATURE:                               ; no features to be set or cleared
ifCLEAR_FEATURE:                                ; no features to be set or cleared
epSYNCH_FRAME:                                  ; not an Isonchronous device
invalid:
;--
xxxGET_PROTOCOL:                                ; not a Boot device
xxxSET_PROTOCOL:                                ; not a Boot device
xxxSET_DESCRIPTOR:         	                    ; descriptors are static
xxxGET_IDLE:                                    ; optional request
xxxSET_IDLE:                                    ; optional request

	;-- display "f"
	;MOV  A, #ledREGf
	;MOV  DPTR, #ledREG
	;MOVX @DPTR, A

	;-- STALL endpoint
	MOV  DPTR, #usbCTRL
	MOV  A, #usbCTRLstall
	MOVX @DPTR, A

	RET

;-------------------------------------------------------------------------------
;* * *   h e l p e r s   * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;-------------------------------------------------------------------------------
devGET_DESCRIPTOR:
	MOV  DPTR, #DevDescr
	MOV  A, #DevDescrLen
	JMP  doGET_DESCRIPTOR

;-------------------
cfgGET_DESCRIPTOR:
	MOV  DPTR, #CfgDescr
	MOV  A, #CfgDescrLen
	JMP  doGET_DESCRIPTOR

;-------------------
strGET_DESCRIPTOR:
	MOV  DPTR, #wValueLO
	MOVX A, @DPTR

	MOV  DPTR, #String0
	JZ   sd

	MOV  DPTR, #String1
	DEC  A
	JZ   sd

	MOV  DPTR, #String2
	DEC  A
	JZ   sd

	MOV  DPTR, #String3
	DEC  A
	JZ   sd

	MOV  DPTR, #String4
sd: MOVX A, @DPTR                               ; length of string
	
	JMP  doGET_DESCRIPTOR

;-------------------------------------------------------------------------------
hidGET_DESCRIPTOR:
	MOV  DPTR, #HidDescr
	MOV  A, #HidDescrLen
	JMP  doGET_DESCRIPTOR

;-------------------
rptGET_DESCRIPTOR:
	MOV  DPTR, #RptDescr
	MOV  A, #RptDescrLen
	JMP  doGET_DESCRIPTOR

;-------------------------------------------------------------------------------
doGET_DESCRIPTOR:
; purpose:
;   send descriptor data to host
; entry: 
;   DPTR - descriptor address
;   A    - descriptor length
; uses:
;   Temp1 - descriptor length, total bytes counter
;   Temp2 - min(wLength, 255), fifo bytes counter
;   Temp3 - LO(descriptor address)
;   Temp4 - HI(descriptor address)
;   Temp5 - LO(destination address)
;   Temp6 - HI(destination address)
;   Temp7 - fifo bytes control
	;-- save parameters
	MOV  Temp1, A
	MOV  Temp3, DPL
	MOV  Temp4, DPH

	;-- compute min(wLength, 255)
	MOV  DPTR, #wLengthHi
	MOVX A, @DPTR
	JZ   sd_lo
	MOV  A, #255
sd_lo:
	MOV  DPTR, #wLengthLo
	MOV  Temp2, A
	MOVX A, @DPTR
	ORL  A, Temp2
	MOV  Temp2, A

	;-- compute min(Temp1, Temp2)
	CLR  C
	SUBB A, Temp1
	JNC  sd_t1
	MOV  A, Temp2
	MOV  Temp1, A
sd_t1:
	
sd_nf:
	;-- compute min(Temp1, usbFIFOlen)
	MOV  A, #usbFIFOlen
	MOV  Temp2, A
	MOV  A, Temp1
	CLR  C
	SUBB A, Temp2
	JNC  sd_ff
	MOV  A, Temp1
	MOV  Temp2, A
	MOV  A, #0
sd_ff:
	MOV  Temp1, A
	MOV  A, Temp2
	MOV  Temp7, A

	;-- setup destination pointer
	MOV  DPTR, #usbFIFO
	MOV  Temp5, DPL
	MOV  Temp6, DPH

sd_nb:
	;-- load data byte
	MOV  DPL, Temp3
	MOV  DPH, Temp4
	MOVX A, @DPTR
	INC  DPTR
	MOV  Temp3, DPL
	MOV  Temp4, DPH

	;-- store data byte
	MOV  DPL, Temp5
	MOV  DPH, Temp6
	MOVX @DPTR, A
	INC  DPTR
	MOV  Temp5, DPL
	MOV  Temp6, DPH

	;-- maintain fifo counter
	MOV  A, Temp2
	DEC  A
	MOV  Temp2, A
	JNZ  sd_nb

	;-- validate fifo
	MOV  DPTR, #usbCTRL
	MOV  A, Temp7
	ORL  A, #usbCTRLread
	MOVX @DPTR, A

	;-- wait for next transaction
	MOV  DPTR, #usbSTAT
sd_io:
	MOVX A, @DPTR
	ANL  A, #usbSTATall
	JZ   sd_io
	CJNE A, #usbSTATdata, sd_x                  ; early status stage

	;-- maintain total counter
	MOV  A, Temp1
	JNZ  sd_nf

	;-- send zero-length package, if required
	MOV  A, #0
	JMP  ControlRead	

sd_x:
	JMP  AcknowledgeStatus

;-------------------------------------------------------------------------------
doGET_STATUS:
	MOV  DPTR, #usbFIFO
	MOVX @DPTR, A

	INC  DPTR
	MOV  A, #0
	MOVX @DPTR, A

	MOV  A, #2
	JMP  ControlRead

;-------------------------------------------------------------------------------
NoDataControl:
	MOV  DPTR, #usbCTRL
	MOV  A, #usbCTRLwrite
	MOVX @DPTR, A
	JMP  WaitForStatus

;-------------------
ControlRead:
	MOV  DPTR, #usbCTRL
	ORL  A, #usbCTRLread
	MOVX @DPTR, A
	JMP  WaitForStatus

;-------------------
WaitForStatus:
	MOV  DPTR, #usbSTAT
w2:	MOVX A, @DPTR
	ANL  A, #usbSTATall
	CJNE A, #usbSTATstatus, w2
;--
AcknowledgeStatus:
	MOV  DPTR, #usbCTRL
	MOV  A, #usbCTRLstatus
	MOVX @DPTR, A
	RET

;-------------------------------------------------------------------------------
doGET_REPORT:
	MOV  DPTR, #ledSL
	MOVX A, @DPTR                               ; left 7-Segment
	MOV  Temp1, A
	INC  DPTR
	MOVX A, @DPTR                               ; right 7-Segment
	MOV  Temp2, A

	MOV  DPTR, #usbFIFO
	MOV  A, Temp1
	MOVX @DPTR, A                               ; report byte #0
	INC  DPTR
	MOV  A, Temp2
	MOVX @DPTR, A                               ; report byte #1

	MOV  A, #2
	JMP  ControlRead

doSET_REPORT:
	MOV  DPTR, #usbFIFO
	MOVX A, @DPTR                               ; report byte #0
	MOV  Temp1, A
	INC  DPTR
	MOVX A, @DPTR                               ; report byte #1
	MOV  Temp2, A

	MOV  DPTR, #ledSL
	MOV  A, Temp1
	MOVX @DPTR, A                               ; left  7-Segment
	INC  DPTR
	MOV  A, Temp2
	MOVX @DPTR, A                               ; right 7-Segment

	JMP AcknowledgeStatus

;-------------------------------------------------------------------------------
;* * *   d a t a   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;-------------------------------------------------------------------------------

;*** device descriptor *********************************************************
DevDescr:
	DB 18, 1                                    ; Length, Type
	DB 10h, 1h                                  ; USB Rev 1.10
	DB 0, 0, 0                                  ; Class, Subclass and Protocol
	DB usbFIFOlen                               ; EP0 size
	DB 0D0h, 0Bh                                ; Vendor ID 0x0BD0: Trenz Electronic
	DB 0, 1                                     ; Product ID 0x0100
	DB 0, 1                                     ; Version Rev 1.0
	DB 1, 2, 0                                  ; Manufacturer, Product & Serial# Names
	DB 1                                        ; #Configs
DevDescrLen EQU $ - DevDescr

;*** config descriptor *********************************************************
CfgDescr:
	DB 9, 2                                     ; Length, Type
	DB LOW(CfgDescrLen), HIGH(CfgDescrLen)
	DB 1, 1, 3                                  ; #Interfaces, Configuration#, Config. Name
	DB 10000000b                                ; Attributes = Bus Powered
	DB 50                                       ; Max. Power is 50x2 = 100mA

;*** interface descriptor ******************************************************
IfDescr:
	DB 9, 4                                     ; Length, Type
	DB 0, 0, 1                                  ; No alternate setting, HID uses EP1
	DB 3                                        ; Class = Human Interface Device
	DB 0, 0                                     ; Subclass and Protocol
	DB 4                                        ; Interface Name

;*** HID descriptor ************************************************************
HidDescr:
	DB 9, 21H                                   ; Length, Type
	DB 0, 1                                     ; HID Class Specification compliance
	DB 0                                        ; Country localization (=none)
	DB 1                                        ; Number of descriptors to follow
	DB 22H                                      ; And it's a Report descriptor
	DB LOW(RptDescrLen), HIGH(RptDescrLen)
HidDescrLen EQU $ - HidDescr

;*** endpoint descriptor *******************************************************
EpDescr:
	DB 7, 5                                     ; Length, Type
	DB 10000001b                                ; Address = IN 1
	DB 00000011b                                ; Interrupt
	DB usbFIFOlen, 0                            ; Maximum packet size (this example only uses 1)
	DB 100                                      ; Poll every 0.1 seconds 
CfgDescrLen EQU $ - CfgDescr

;*** report descriptor *********************************************************
RptDescr:
	DB 6, 0, 0FFH                               ; Usage_Page (Vendor Defined)
	DB 9, 1                                     ; Usage (I/O Device)
	DB 0A1H, 1                                  ; Collection (Application)
;	input report                                ; --
	  DB 19H, 1                                 ;   Usage_Minimum
	  DB 29H, 2                                 ;   Usage_Maximum
	  DB 15H, 0                                 ;   Logical_Minimum (0)
	  DB 26H, 255, 0                            ;   Logical_Maximum (255)
	  DB 75H, 8                                 ;   Report_Size  (8 bits)
	  DB 95H, 2                                 ;   Report_Count (2 fields) 
	  DB 81H, 2                                 ;   Input (Data,Var,Abs)
;	output report                               ; --
	  DB 19H, 1                                 ;   Usage_Minimum
	  DB 29H, 2                                 ;   Usage_Maximum
	  DB 15H, 0                                 ;   Logical_Minimum (0)
	  DB 26H, 255, 0                            ;   Logical_Maximum (255)
	  DB 75H, 8                                 ;   Report_Size  (8 bits)
	  DB 95H, 2                                 ;   Report_Count (2 fields) 
	  DB 91H, 2                                 ;   Output (Data,Var,Abs)
	DB 0C0H                                     ; End_Collection
RptDescrLen EQU $ - RptDescr

;*** string #0 *****************************************************************
String0:                                        ; declare the UNICODE strings
	DB 4, 3, 9, 4                               ; english strings
;String0Len EQU $ - String0

;*** string #1 *****************************************************************
String1:                                        ; Manufacturer
	DB (String2-String1), 3                     ; Length, Type 
	DB 'T',0,'r',0,'e',0,'n',0,'z',0,' ',0,
	DB 'E',0,'l',0,'e',0,'c',0,'t',0,'r',0,'o',0,'n',0,'i',0,'c',0

;*** string #2 *****************************************************************
String2:                                        ; Product Name
	DB (String3-String2), 3
	DB 'T',0,'E',0,'-',0,'U',0,'S',0,'B',0

;*** string #3 *****************************************************************
String3:                                        ; Configuration Name
	DB (String4-String3), 3
	DB 'A',0,'p',0,'p',0,'l',0,'i',0,'c',0,'a',0,'t',0,'i',0,'o',0,'n',0,' ',0,
	DB 'N',0,'o',0,'t',0,'e',0

;*** string #4 *****************************************************************
String4:                                        ; Interface Name
	DB (EndOfDescriptors-String4), 3
	DB 'H',0,'I',0,'D',0,' ',0,'E',0,'x',0,'a',0,'m',0,'p',0,'l',0,'e',0
EndOfDescriptors:
	DW 0                                        ; Backstop for String Descriptors

;-------------------------------------------------------------------------------
	END
