--------------------------------------------------------------------------------
-- Copyright (c) 2000 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--     
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--     
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--     
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--------------------------------------------------------------------------------
-- Project:      Application Note: SoC with Xilinx Virtex
-- File:         xsvPADS.vhd
-- Description:  FPGA Pin Mapping.
-- Version:      FB, 2001jan30
--------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

-- synopsys translate_off 
library VIRTEX;
-- synopsys translate_on 

entity xsvPads is
	port(
		clk_p89:     in  STD_LOGIC;
		sw1_p174:    in  STD_LOGIC;
		-- Philips PDIUSBP11A USB Transceiver
		rcv_p11:     in  STD_LOGIC;
		vp_p10:      in  STD_LOGIC;
		vm_p9:       in  STD_LOGIC;
		vpo_p13:     out STD_LOGIC;
		fse0_p17:    out STD_LOGIC;
		oe_p12:      out STD_LOGIC;
		suspnd_p176: out STD_LOGIC;
		-- 7-Segment left
		sl0_p177:    out STD_LOGIC;
		sl1_p167:    out STD_LOGIC;
		sl2_p163:    out STD_LOGIC;
		sl3_p156:    out STD_LOGIC;
		sl4_p145:    out STD_LOGIC;
		sl5_p138:    out STD_LOGIC;
		sl6_p134:    out STD_LOGIC;
		-- 7-Segment right
		sr0_p124:    out STD_LOGIC;
		sr1_p132:    out STD_LOGIC;
		sr2_p133:    out STD_LOGIC;
		sr3_p139:    out STD_LOGIC;
		sr4_p141:    out STD_LOGIC;
		sr5_p144:    out STD_LOGIC;
		sr6_p147:    out STD_LOGIC
		);
end xsvPads;

--------------------------------------------------------------------------------
architecture BHV of xsvPads is
	component bufg
		port (
			I : in std_ulogic;
			O : out std_ulogic
			);
	end component;
	
	component inv
		port (
			I : in std_ulogic;
			O : out std_ulogic
			);
	end component;
	
	component xsvCore
		port (
			clk48 : in STD_LOGIC;
			rcv : in STD_LOGIC;
			rst : in STD_LOGIC;
			vm : in STD_LOGIC;
			vp : in STD_LOGIC;
			fse0 : out STD_LOGIC;
			oe : out STD_LOGIC;
			sl : out STD_LOGIC_VECTOR (6 downto 0);
			sr : out STD_LOGIC_VECTOR (6 downto 0);
			suspnd : out STD_LOGIC;
			vpo : out STD_LOGIC
			);
	end component;
	
	signal CLK48 : STD_LOGIC;
	signal RST : STD_LOGIC;
	signal SL : STD_LOGIC_VECTOR (6 downto 0);
	signal SR : STD_LOGIC_VECTOR (6 downto 0);
	
	-- synopsys translate_off
	for U2 : inv use entity VIRTEX.inv;
	for U1 : bufg use entity VIRTEX.bufg;
	-- synopsys translate_on
	
begin
	
	U1 : bufg
	port map(
		I => clk_p89,
		O => clk48
		);
	
	U2 : inv
	port map(
		I => sw1_p174,
		O => rst
		);
	
	Ucore : xsvCore
	port map(
		clk48 => clk48,
		fse0 => fse0_p17,
		oe => oe_p12,
		rcv => rcv_p11,
		rst => rst,
		sl => sl,
		sr => sr,
		suspnd => suspnd_p176,
		vm => vm_p9,
		vp => vp_p10,
		vpo => vpo_p13
		);
	
	sl0_p177 <= sl(0);
	sl1_p167 <= sl(1);
	sl2_p163 <= sl(2);
	sl3_p156 <= sl(3);
	sl4_p145 <= sl(4);
	sl5_p138 <= sl(5);
	sl6_p134 <= sl(6);
	
	sr0_p124 <= sr(0);
	sr1_p132 <= sr(1);
	sr2_p133 <= sr(2);
	sr3_p139 <= sr(3);
	sr4_p141 <= sr(4);
	sr5_p144 <= sr(5);
	sr6_p147 <= sr(6);
	
end BHV;

--------------------------------------------------------------------------------
-- end of file
